﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

namespace HeuristicLab.Problems.Instances.QAPGenerator {
  public sealed class TaibQAPInstanceGeneratorDescriptor : QAPInstanceGeneratorDescriptor {

    private int maxClusterDistance;
    public int MaxClusterDistance {
      get { return maxClusterDistance; }
      set {
        if (maxClusterDistance == value) return;
        maxClusterDistance = value;
        OnPropertyChanged("MaxClusterDistance");
      }
    }

    private int maxClusterSize;
    public int MaxClusterSize {
      get { return maxClusterSize; }
      set {
        if (maxClusterSize == value) return;
        maxClusterSize = value;
        OnPropertyChanged("MaxClusterSize");
      }
    }

    private int maxClusterRadius;
    public int MaxClusterRadius {
      get { return maxClusterRadius; }
      set {
        if (maxClusterRadius == value) return;
        maxClusterRadius = value;
        OnPropertyChanged("MaxClusterRadius");
      }
    }

    private int minFlowExponent;
    public int MinFlowExponent {
      get { return minFlowExponent; }
      set {
        if (minFlowExponent == value) return;
        minFlowExponent = value;
        OnPropertyChanged("MinFlowExponent");
      }
    }

    private int maxFlowExponent;
    public int MaxFlowExponent {
      get { return maxFlowExponent; }
      set {
        if (maxFlowExponent == value) return;
        maxFlowExponent = value;
        OnPropertyChanged("MaxFlowExponent");
      }
    }

    public override string Name {
      get { return "Tai-b Instances"; }
    }

    public override string Description {
      get { return "Generates randomly clustered symmetric QAP instances with more real-world like flows."; }
    }

    public override bool IsConfigurationValid {
      get {
        return base.IsConfigurationValid
          && MaxClusterDistance > 0
          && MaxClusterRadius > 0
          && MaxClusterSize > 0
          && MinFlowExponent < MaxFlowExponent
          && MaxFlowExponent > 0;
      }
    }

    public TaibQAPInstanceGeneratorDescriptor() {
      // default ctor should result in a valid configuration
      N = 20;
      Seed = 123456789;
      MaxClusterDistance = 1000;
      MaxClusterRadius = 20;
      MaxClusterSize = 10;
      MinFlowExponent = -5;
      MaxFlowExponent = 3;
    }

    public override QAPData GetInstance() {
      double[,] A, B;
      TaillardQAPInstanceGenerator.GenerateTaib(N, maxClusterDistance, maxClusterSize, maxClusterRadius, minFlowExponent, maxFlowExponent, out A, out B, new Bratley(Seed));
      return new QAPData {
        Name = "tai" + N.ToString("00") + "b (" + maxClusterDistance + ", " + maxClusterSize + ", " + maxClusterRadius + ") [" + minFlowExponent + ";" + maxFlowExponent + "]",
        Description = "Instance generated with TaillardQAPInstanceGenerator.GenerateTaib",
        Dimension = N,
        Weights = B,
        Distances = A
      };
    }
  }
}
